############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus import utilc
from pyGlobus.util import GlobusException
from pyGlobus.ftpControl import FtpControl
from pyGlobus.ftpControl import Parallelism
from pyGlobus.ftpControl import HostPortArray
from pyGlobus.ftpControl import TcpBuffer
from pyGlobus.ftpControl import DCAU
from pyGlobus.ftpControl import FtpControlException
from pyGlobus.security import GSSCred
import unittest,ConfigParser
from pyGlobus import ftpControlc
from pyGlobus import ftpControl


config = None
config_file = 'config.py'

class FtpControlException(GlobusException):
    pass

class FtpControlTest__basic(unittest.TestCase):
    def setUp(self):
	global config
	if config == None:
	    config = ConfigParser.ConfigParser()
	    config.read(config_file)
	self._ftpServer = FtpControl()
	
    def tearDown(self):
	print "DONE"
	
    def check_local_send_eof(self):
	print "check_local_send_eof"
	eof = 1
	self._ftpServer.local_send_eof(eof)
			
    def check_local_parallelism(self):
	print "check_local_parallelism"
	parallel = Parallelism()
	md = ftpControl.PARALLELISM_FIXED
	parallel.set_mode(md)
        self._ftpServer.local_parallelism(parallel)
    
    def check_local_mode(self):
	"""There is a pesky valgrind UMR bug to track down in this method MKR """
	print "check_local_mode"
        #only STREAM and EXTENDED_MODE are supported for this call in the current
        #implementation
        mode = 'S'
        self._ftpServer.local_mode(mode)
	
    def check_local_tcp_buffer(self):
	print "check_local_tcp_buffer"
        tcpBuf = TcpBuffer()
        size = 100
        tcpBuf.set_fixed(size)
        self._ftpServer.local_tcp_buffer(tcpBuf)
	
    def check_local_dcau(self):
	print "check_local_dcau"
	dcau = DCAU()
	mode = ftpControl.DCAU_SELF
	dcau.set_mode(mode)
	cred = GSSCred()
	self._ftpServer.local_dcau(dcau, cred)
	
    def check_local_pbsz(self):
	print "check_local_pbsz"
        size = 100
        self._ftpServer.local_pbsz(size)
        #def response_callback(arg,handle, error, response):
        #print "in response callback"
        #def check_parallelism_init(self):
        #print "in check_parallelism_init"
        #parallel = Parallelism()
	
    def check_local_spas(self):
        print "check_local_spas"
        size = 10
        hp_array = HostPortArray(size)
	try:
	    self._ftpServer.local_spas(hp_array)
	    
        except GlobusException, ex:
            if ex.__str__() != "globus_ftp_control_local_spas(): this function is not implemented":
                raise FtpControlException(ex.__str__())
            return
	
    def check_local_spor(self):
	print "check_local_spor"
        size = 10
        hp_array = HostPortArray(size)
	try:
	    self._ftpServer.local_spor(hp_array)        
	except GlobusException, ex:
            if ex.__str__() != "globus_ftp_control_local_spas(): this function is not implemented":
                raise FtpControlException(ex.__str__())
            return

    def implement_check_data_connect(self):
	print "check_data_connect"

    def implement_check_data_write(self):
	print "check_data_write"

    def implement_check_local_pasv(self):
	print "check_local_pasv"

    def implement_check_local_port(self):
	print "check_local_port"

    def implement_check_read_commands(self):
	print "check_read_commands"

    def implement_check_send_response(self):
	print "check_send_response"

    def implement_check_raw_command(self):
	print "check_raw_command"
    
    def implement_check_code(self):
	print "check_code"

    def implement_check_operation(self):
	print "check_operation"

    def implement_check_create_data_info(self):
	print "check_create_data_info"

    def implement_check_release_data_info(self):
	print "check_release_data_info"
    
    def implement_check_data_write_stripe(self):
	print "check_data_write_stripe"

    def implement_check_x_data_write_stripe(self):
	print "check_x_data_write_stripe"

    def implement_check_authenticate(self):
	print "check_authenticate"
	
    def implement_check_send_command(self):
	print "check_send_command"

    def implement_check_abort(self):
	print "check_abort"
	
    def implement_check_quit(self):
        #self._ftpServer.quit(self.quit_callback,0)	
	a = 1

    def implement_check_force_close(self):
	print "check_force_close"

    def implement_check_data_connect_read(self):
	print "check_data_connect_read"

    def implement_check_data_add_channels(self):
	print "check_data_add_channels"

    def implement_check_data_send_eof(self):
	print "check_data_send_eof"

    def implement_check_data_remove_channels(self):
	print "check_data_remove_channels"

    def implement_check_data_query_channels(self):
	print "check_data_query_channels"

    def implement_check_get_total_data_channels(self):
	print "check_get_total_data_channels"

    def implement_check_data_read(self):
	print "check_data_read"

    def implement_check_local_stru(self):
	print "check_local_stru"
	
    
	
	
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(FtpControlTest__basic, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
