############################################################################
# Joshua R. Boverhof, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import sys, unittest, ConfigParser, time
import SocketServer
from threading import *

from pyGlobus import utilc
from pyGlobus.io import GSITCPSocket,GSITCPSocketServer,TCPIOAttr,AuthData
from pyGlobus import ioc
from pyGlobus.util import Buffer,GlobusException

config = None
g_port = 0

def auth_callback(arg, handle, identity, context):
        return 1
    
class testServer(Thread):
    def setup(self, port):
        global g_port
        g_port = port
        self.handle = GSITCPSocketServer(('',g_port), GSIRequestHandler)
        self.setDaemon(1)
        return self.handle

    def run(self):
        self.handle.serve_forever()

    def server_close(self):
        self.handle.server_close()

class GSIRequestHandler(SocketServer.StreamRequestHandler):
   """A minimalistic Request handler class that receives data from the
   client only. It prints it all out and then exits.
   """
   def handle(self):
      result = 'X'
      print "GSIRequestHandler got a connection from %s" \
            % (self.client_address,)

      result = self.rfile.readline()
      print result
      while result:
         result = self.rfile.readline()
         print result

class testClient(Thread):
    def run(self):
        host = "localhost"
        try:
            attr = TCPIOAttr()
            attr.set_authentication_mode(ioc.GLOBUS_IO_SECURE_AUTHENTICATION_MODE_GSSAPI)
            authData = AuthData()
            attr.set_authorization_mode(ioc.GLOBUS_IO_SECURE_AUTHORIZATION_MODE_SELF, authData)
            attr.set_channel_mode(ioc.GLOBUS_IO_SECURE_CHANNEL_MODE_GSI_WRAP)
            attr.set_delegation_mode(ioc.GLOBUS_IO_SECURE_DELEGATION_MODE_LIMITED_PROXY)
            soc = GSITCPSocket()
            soc.connect(host, g_port, attr)
	    addrList = soc.get_remote_address()
	    str = "Foobar, fOobar, foObar, fooBar, foobAr, foobaR"
            nBytes = soc.write(str, len(str))
        except GlobusException, ex:
            print "caught an exception"
            print ex.args
            sys.exit(-1)
            return

class GSITCPSocketServerCase(unittest.TestCase):
    
    def setUp(self):
        print "Begin GSITCPSocketServer"
        self.server_t = testServer()
        self.client_t = testClient()

    def checkGSITCPSocketServer(self):
        self.server_t.setup(14831)
        time.sleep(1)
        self.server_t.start()
        time.sleep(1)
        self.client_t.start()
        time.sleep(1)

    def tearDown(self):
#        self.server_t.server_close()
        print "done with GSITCPSocketServer"
        
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSITCPSocketServerCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")

if __name__ == "__main__" : main()
