############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

import ConfigParser, unittest
from pyGlobus.security import *
from pyGlobus.util import GlobusException

"""
Unittest for the GSSContext module.
"""

config = None # ConfigParser object
g_ctx_blob = None


class GSSContextTestCase(unittest.TestCase):
    """
       Test case for the GSSContext module.
       """
    
    def setUp(self):
        global config
        
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.py")
        self.__cred = GSSCred()
            
    def tearDown(self):
        del self.__cred
    
    def checkGSSContext(self):
        try:
            ctx = GSSContext()
            del ctx
            print "\nGSSContextTestCase completed\n"
        except GlobusException, ex:
            print "Exception occured during testing."
            assert 0, ex.args
        return

    def checkInitContext(self):
        global g_ctx_blob
        print "\ncheckInitContext\n"
        
        context = GSSContext()
        self.__cred.acquire_cred()
        print "done with init ctx"
        lifetime, cred_name = self.__cred.inquire_cred()
        print "Cred info:  lifetime: " + str(lifetime) + "\nName: ", cred_name.display()
        buf = GSSBuffer()
        name = GSSName()
        t = context.init_context(self.__cred, name,buf)
        print t
        major = t[0]
        minor = t[1]
        print "major is: ", t[0]
        print "minor is: ", t[1]
        print "ctx_blob is: ", t[2]
        g_ctx_blob = t[2]
        if major != gsic.GSS_S_CONTINUE_NEEDED and major != gsic.GSS_S_COMPLETE:
            list = gsic.display_status(major, minor)
            ex = apply(GSSContextException, list)
            raise ex
        else:
            print "Passed init_context"
        
        del context
        del buf
        del name
        
    def checkTheAcceptContext(self):
        print "checkAcceptContext\n"

        try:
            
            ctx  = GSSContext()
            try:
                self.__cred.acquire_cred()
            except GSSCredException, ex:
                print ex
                print "Exception during acquire_cred"
                raise ex
            buf = GSSBuffer()
            buf.set_buffer(g_ctx_blob)
            print "ctx_blob",g_ctx_blob
            major,minor,token_string = ctx.accept_context(self.__cred,buf)
            print major
            print minor
            print token_string
            if major != gsic.GSS_S_CONTINUE_NEEDED and major != gsic.GSS_S_COMPLETE:
                list = gsic.display_status(major, minor)
                ex = apply(GSSContextException, list)
                raise ex
            else:
                print "Passed accept_context"

        except GlobusException, ex:
            print ex
            print "Exception occurred during accept_context test"
      
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GSSContextTestCase, "check"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

