############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################

from pyGlobus.gramClient import GramClient, JOB_STATE_PENDING, JOB_STATE_ACTIVE
from pyGlobus.gramClient import JOB_STATE_FAILED, JOB_STATE_DONE, JOB_STATE_ALL
from pyGlobus.gramClient import GramClientAttr
from pyGlobus.util import GlobusException
import sys, ConfigParser, unittest
from threading import *
from pyGlobus.security import *

config = None # ConfigParser object

def func2(userArg, contact, state, error):
    print "Got callback from register_job"
    print "Contact is: %s, State is: %i, Error is: %i" % (contact, state, error)
    return

def func(cv, contact, state, error):
    if state == JOB_STATE_PENDING:
        print "Job is pending"
    elif state == JOB_STATE_ACTIVE:
        print "Job is active"
    elif state == JOB_STATE_FAILED:
        print "Job failed"
        cv.acquire()
        cv.notify()
        cv.release()
    elif state == JOB_STATE_DONE:
        print "Job is done"
        cv.acquire()
        cv.notify()
        cv.release()


#############################################################################
#  Unit test for the GramClient Module:
#  Given a remote machine and an rsl command, the test function creates a
#  GramClient object and which sends the rsl message to the remote machine
#  and if all goes well then the command is executed.
#
##############################################################################


class GramClientTestCase(unittest.TestCase):
    "Test case for the GramClient module"
    
    def checkGramClient(self):
        global config
        print  "\nGramClientTestCase begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.py")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl")
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.submit_request(rm, rsl, JOB_STATE_ALL,
                                                                callbackContact)
            
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\nGramClientTestCase finished\n"
        except GlobusException, ex:
            assert 0, ex.args

    def checkGramClientNonBlocking(self):
        global config
        print  "\nGramClientNonBlockingTestCase begin.\n"
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read("config.py")
            
        rm = config.get("gramClient", "rm")
        rsl = config.get("gramClient", "rsl")
        condV = Condition(Lock())
        try:
            gramClient = GramClient()
            assert not gramClient == None, "Error creating the GramClient object"
            gcAttr = GramClientAttr()
            assert not gcAttr == None, "Error creating the GramClientAttr object"
            callbackContact = gramClient.set_callback(func, condV)
            condV.acquire()
            jobContact = gramClient.register_submit_request(rm, rsl, 
                                                        JOB_STATE_ALL, callbackContact,
                                                        gcAttr, func2, None)
           
            condV.wait()
            condV.release()
            gramClient.remove_callback(callbackContact)
            print "\nGramClientNonBlockingTestCase finished\n"
        except GlobusException, ex:
            print ex
            assert 0, ex.args

class GramClientAttrTestCase(unittest.TestCase):
    def TestGetSets(self):
	self.cred = GSSCred()

        print "GramClientAttrTestCase begin--"

	try:
	    gca  = GramClientAttr()
	    gca.set_credential(self.cred)  # tests set_credential()
	except GlobusException, s:
	    if str(s)=="globus_gram_client_attr_set_credential is unimplemented in this version of gram_client":
		print 'expected'
	    else:
	    	print 'unexpected!'
		assert 0, str(s)

	try: gca.get_credential()  # tests get_credential()
	except GlobusException, s:
		if str(s)=="globus_gram_client_attr_get_credential is unimplemented in this version of gram_client":
		    print 'expected'
	   	else:
		    print str(s)

	print "GramClientAttrTestCase end--"


def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(GramClientTestCase, "check"))
    suite.addTest(unittest.makeSuite(GramClientAttrTestCase,"Test"))
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
    
                  

