############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################
from pyGlobus import rlsClientc
from pyGlobus.rlsClient import RlsClient
from pyGlobus.rlsClient import RlsAttr
from pyGlobus.rlsClient import RlsClientException
from pyGlobus.rlsClient import RlsAttrException
import unittest,ConfigParser, time

config = None
config_file ="config.py"

class RlsClientTest(unittest.TestCase):
    """
       Parent class which has the overrides the setUp and
       tearDown methods that all of the children testing classes use
       """

    def setUp(self):
	global config
        if config == None:
            config = ConfigParser.ConfigParser()
            config.read(config_file)
	url = config.get("rlsClient", "url")
	print "in setUp"
	self._client = RlsClient(url)

    def tearDown(self):
	del self._client
	print "DONE"

class RlsClientTestCaseCreate(RlsClientTest):
    """Unittest for the rlsClient module"""
    
    def check_lrc_create_lfn(self):
	print "testing lrc_create_lfn"
	lfn = config.get("rlsClient", "lfn")
	pfn = config.get("rlsClient", "pfn1")
	try:
	    self._client.lrc_create_lfn(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex

class RlsClientTestCaseTest(RlsClientTest):

    def check_lrc_add(self):
	print "testing lrc_add"
	lfn = config.get("rlsClient", "lfn")
	pfn = config.get("rlsClient", "pfn2")
	try:
	    self._client.lrc_add(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex    


    def check_lrc_get_lfn_wc(self):
	print "testing lrc_get_lfn_wc"
	regExp = config.get("rlsClient", "regExp")
	try:
	    dict = self._client.lrc_get_lfn_wc(regExp)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex
	     return
	print dict

    def check_lrc_get_pfn_wc(self):
	print "testing lrc_get_pfn_wc"
	regExp = config.get("rlsClient", "regExp")
	try:
	    dict = self._client.lrc_get_pfn_wc(regExp)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex
	     return
	print dict
	
    def check_lrc_get_lfn(self):
	print "testing lrc_get_lfn"
	pfn = config.get("rlsClient", "pfn1")
	try:
	    list = self._client.lrc_get_lfn(pfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	print list

    def check_lrc_exists(self):
	print "testing lrc_exists"
	fn = config.get("rlsClient", "badFn")
	try:
	    exists = self._client.lrc_exists(fn,rlsClientc.globus_rls_obj_lrc_pfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	if not exists:
	    print "Test PASSED"

class LrcRlsAttrTestCase(RlsClientTest):

    def setUp(self):
	RlsClientTest.setUp(self)
	self._attr = RlsAttr()
	self._attr.set_name("testAttr")
	self._attr.set_type(rlsClientc.globus_rls_attr_type_str)
	self._attr.set_objtype(rlsClientc.globus_rls_obj_lrc_lfn)
	self._attr.set_val("Attribute string")
	try:
	    self._client.lrc_attr_create(self._attr)
	except RlsClientException, ex:
	    print "Caught Exception in setUp"
	    print ex

	lfn = config.get("rlsClient", "lfn")
	try:
	    self._client.lrc_attr_add(lfn, self._attr)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    
	print "attribute added"
	
    def tearDown(self):
	lfn = config.get("rlsClient", "lfn")
	try:
	    self._client.lrc_attr_remove(lfn, self._attr)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	print "attribute removed"
	try:
	    self._client.lrc_attr_delete(self._attr)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	RlsClientTest.tearDown(self)
	

    def checkLrcAttrSearch(self):
	print "testing lrc_attr_search"
	attr = RlsAttr()
	attr.set_name("testAttr")
	attr.set_objtype(rlsClientc.globus_rls_obj_lrc_lfn)
	attr.set_val("testAttr")
	lfn =  config.get("rlsClient", "lfn") 
	try:
	    dict = self._client.lrc_attr_search(attr,rlsClientc.globus_rls_attr_op_all)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    #return
	print "Search dictionary: ", dict
	

    def checkLrcAttrGet(self):
	print "checkLrcAttrGet"
	objtype = self._attr.get_objtype()
	name = self._attr.get_name()
	try:
	    list = self._client.lrc_attr_get(objtype, name)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex   
	    return
	for attr in list:
	    print "attr name: ", attr.get_name()

    def checkLrcAttrGetValue(self):
	print "checkLrcAttrGetValue"
	objtype = self._attr.get_objtype()
	name = self._attr.get_name()
	fn =  config.get("rlsClient", "lfn")

	try:
	    list = self._client.lrc_attr_value_get(fn,objtype, name)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex   
	    return
	for attr in list:
	    print "attr name: ", attr.get_name()
    

	
class RlsClientTestCaseClear(RlsClientTest):

    def check_lrc_delete(self):
	print "testing lrc_delete"
	pfn = config.get("rlsClient", "pfn1")
	lfn =  config.get("rlsClient", "lfn")
	try:
	    self._client.lrc_delete(lfn,pfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return

    def check_lrc_clear(self):
	print "Test lrc_clear"
	try:
	    self._client.lrc_clear()
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex

class RlsClientTestCaseRli(RlsClientTest):

    def setUp(self):
	RlsClientTest.setUp(self)
	url = config.get("rlsClient", "url")
	try:
	    self._client.lrc_rli_add(url, "", 1)
	except RlsClientException, ex:
	   print "Caught Exception in setUp"
	   print ex

    def tearDown(self):
	url = config.get("rlsClient", "url")
	try:
	    self._client.lrc_rli_delete(url)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	RlsClientTest.tearDown(self)
	

    def checkLrcRliList(self):
	print "testing checkLrcRliList"
	try:
	    dict = self._client.lrc_rli_list()
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex

	print "dictionary: ", dict

    def checkLrcRliInfo(self):
	print "testing checkLrcRliInfo"
	url = config.get("rlsClient", "url")
	try:
	    dict = self._client.lrc_rli_info(url)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex

	print "dictionary: ", dict

    def checkLrcRliGetPart(self):
	print "testing checkLrcRliGetPart"
	try:
	    list = self._client.lrc_rli_get_part()
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex

	print "list is: ",list

    def checkLrcRliExists(self):
	print "testing checkLrcRliExists"
	url = config.get("rlsClient", "url")
	
	lfn = "testLfn"
	pfn = "/etc/shells"
	try:
	    self._client.lrc_add(lfn, pfn)
	except RlsClientException, ex:
	     print "Caught Exception"
	     print ex  

	print "lfn Added"
	
	exists = 0
	try:
	    exists = self._client.rli_lfn_exists("testLfn")
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex

	if exists == 1:
	    print "Test PASSED"
	else:
	    print "Test Failed"

	try:
	    self._client.lrc_delete(lfn,pfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	print "lfn deleted"

    def checkRliGetLrc(self):
	print "testing checkRliGetLrc"
	lfn = "testLfn"
	try:
	    list = self._client.rli_get_lrc(lfn)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	print "RLI LRC list", list

    def checkRliGetLrcWc(self):
	print "testing  checkRliGetLrcWc"
	pattern = '*'
	try:
	    dict = self._client.rli_get_lrc(pattern)
	except RlsClientException, ex:
	    print "Caught Exception"
	    print ex
	    return
	print "RLI LRC dict", dict
	
class RlsAttrTestCase(unittest.TestCase):

    def setUp(self):
	self._attr = RlsAttr()

    def tearDown(self):
	print "DONE"
	
    def checkSetAndGetName(self):
	print "testing checkSetAndGetName" 
	name = "testAttr"
	self._attr.set_name(name)
	newName = self._attr.get_name()
	if name == newName:
	    print "Test Passed"

    def checkSetAndGetObjType(self):
	print "testing checkSetAndGetObjType"
	type = rlsClientc.globus_rls_obj_lrc_lfn
	self._attr.set_type(type)
	newType = self._attr.get_type()
	if type == newType:
	    print "Test Passed"
	
    def checkSetAndGetVal(self):
	print "testing checkSetAndGetVal"
	valStr = "Cream"
	self._attr.set_val(valStr)
	newValStr = self._attr.get_val()
	if valStr == newValStr:
	    print "Setting String val works"

	valFloat = 1.5
	self._attr.set_val(valFloat)
	newValFloat = self._attr.get_val()
	if valFloat == newValFloat:
	    print "Setting Float works"


	valInt = 5
	self._attr.set_val(valInt)
	newValInt = self._attr.get_val()
	if valInt == newValInt:
	    print "Setting Int works"
	    

    def checkSetAndGetValTime(self):
	print "checkSetAndValTime"
	try:
	    valTime = time.localtime()
	    vTime = tuple(valTime)
	    self._attr.set_val(vTime)
	    newValTime = self._attr.get_val()
	    
	    if valTime == newValTime:
		print "Setting time works"
	    
	except RlsAttrException, ex:
	    print "Exception caught"
	    print ex
	    
def makeTestSuite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(RlsClientTestCaseCreate, "check"))
    suite.addTest(unittest.makeSuite(RlsClientTestCaseTest, "check"))
    suite.addTest(unittest.makeSuite(LrcRlsAttrTestCase, "check"))
    suite.addTest(unittest.makeSuite(RlsAttrTestCase, "check"))
    suite.addTest(unittest.makeSuite(RlsClientTestCaseRli, "check"))
    suite.addTest(unittest.makeSuite(RlsClientTestCaseClear, "check"))
    
    return suite

def main():
    unittest.main(defaultTest="makeTestSuite")


if __name__ == "__main__" : main()
   
 
    
