/*############################################################################
# Keith R. Jackson, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gramClientc
%{
#include "globus_gram_client.h"
%}

%include ./src/pyGlobus-typemaps.i

%apply char **OUTPUT { const char **callback_contact,
			   char **callback_contact };

%apply int *OUTPUT { int* job_status, int *failure_code };

%{
static PyThreadState *thread_state = NULL;

typedef struct {
    PyObject* callback;
    PyObject* arg;
} callbackStruct;

static void PythonCallback(void* user_data, char* job_contact,
                           int state, int errorcode) {
    PyObject *func, *arglist, *result, *arg;
    callbackStruct* cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(thread_state);
    func = cbStruct->callback;
    arg = cbStruct->arg;
    arglist = Py_BuildValue("(Osii)", arg, job_contact, state, errorcode);
    result = PyEval_CallObject(func, arglist);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(thread_state);
    return; 
}
static void
NonBlockingPythonCallback(void *user_data, 
                globus_gram_protocol_error_t failure_code, 
                const char *job_contact,
                globus_gram_protocol_job_state_t job_state,
                globus_gram_protocol_error_t job_failure_code) {
    PyObject *func, *arglist, *result, *arg;
    callbackStruct *cbStruct = (callbackStruct*)user_data;
    PyEval_AcquireThread(thread_state);
    func = cbStruct->callback;
    arg = cbStruct->arg;
    arglist = Py_BuildValue("(Osii)", arg, job_contact, (int)job_state,
                                                    (int)failure_code);
    result = PyEval_CallObject(func, arglist);
    Py_DECREF(arglist);
    Py_XDECREF(result);
    PyEval_ReleaseThread(thread_state);
    return;
}
%}

%inline %{
globus_module_descriptor_t* get_module(void) {
    return GLOBUS_GRAM_CLIENT_MODULE;
}

PyObject*
callback_allow(PyObject *pyfunc, PyObject *arg, char ** callback_contact) {
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    if (thread_state == NULL) {
        PyEval_InitThreads();
        thread_state = PyThreadState_New(PyThreadState_Get()->interp);
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_callback_allow(PythonCallback, cbStruct,
                                           callback_contact);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list; 
}

int
ping(char * resource_manager_contact) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_ping(resource_manager_contact);
    Py_END_ALLOW_THREADS
    return ret;
}

PyObject*
job_request(char * resource_manager_contact, const char * description,
            const int job_state_mask, const char * callback_contact) {
    int ret;
    PyObject* tuple;
    char* job_contact;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "job_request: PyTuple_New()");
        return NULL;
    }
    ret = globus_gram_client_job_request(resource_manager_contact,
                                         description, job_state_mask,
                                         callback_contact, &job_contact);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else {
	PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", job_contact));
	globus_gram_client_job_contact_free(job_contact);
    }
    return tuple;
}

int
job_status(char* job_contact, int* job_status, int* failure_code) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_status(job_contact, job_status, failure_code);
    Py_END_ALLOW_THREADS
    return ret;
}

int
job_cancel(char* job_contact) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_cancel(job_contact);
    Py_END_ALLOW_THREADS
    return ret;
}

int
job_refresh_credentials(char *job_contact, gss_cred_id_t *cred) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_refresh_credentials(job_contact, *cred);
    Py_END_ALLOW_THREADS
    return ret;
}

PyObject*
register_job_refresh_credentials(char* job_contact, gss_cred_id_t *cred, 
                  globus_gram_client_attr_t *gattr, PyObject *pyfunc,
                  PyObject *arg) {
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc;
    cbStruct->arg = arg;
    if (thread_state == NULL) {
        PyEval_InitThreads();
        thread_state = PyThreadState_New(PyThreadState_Get()->interp);
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_refresh_credentials(job_contact, 
                                        *cred, *gattr, NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list; 
}

PyObject*
register_job_request(const char* rm_contact, const char *description,
                                        int job_state_mask, 
                                        const char *callback_contact,
                                        globus_gram_client_attr_t *gattr,
                                        PyObject *pyfunc, PyObject *arg) { 
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    if (thread_state == NULL) {
        PyEval_InitThreads();
        thread_state = PyThreadState_New(PyThreadState_Get()->interp);
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_request(rm_contact, description,
                                        job_state_mask, callback_contact,
                                        *gattr, NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list;
}

PyObject*
register_callback_registration(const char* job_contact, int job_state_mask, 
                                        const char *callback_contact,
                                        globus_gram_client_attr_t *gattr,
                                        PyObject *pyfunc, PyObject *arg) { 
    int ret;
    PyObject *list, *cbPtr;
    callbackStruct* cbStruct;
    cbStruct = (callbackStruct*)globus_malloc(sizeof(callbackStruct));
    Py_XINCREF(pyfunc);
    Py_XINCREF(arg);
    cbStruct->callback = pyfunc; 
    cbStruct->arg = arg;   
    if (thread_state == NULL) {
        PyEval_InitThreads();
        thread_state = PyThreadState_New(PyThreadState_Get()->interp);
    }
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_register_job_callback_registration(job_contact, 
                                        job_state_mask, callback_contact,
                                        *gattr, NonBlockingPythonCallback, cbStruct);
    Py_END_ALLOW_THREADS
    cbPtr = SWIG_NewPointerObj((void*)cbStruct, SWIGTYPE_p_callbackStruct, 0);
    list = PyList_New(0);
    PyList_Append(list, Py_BuildValue("i", ret));
    PyList_Append(list, cbPtr);
    return list;
}

void
free_nb_callback(callbackStruct *cbStruct) {
    int ret;
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
    return;
}

int
remove_callback(char* callback_contact, callbackStruct* cbStruct) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_callback_disallow(callback_contact);
    Py_END_ALLOW_THREADS
    Py_XDECREF(cbStruct->callback);
    Py_XDECREF(cbStruct->arg);
    globus_free(cbStruct);
    return ret;
}

int
callback_register(char* job_contact, const int job_state_mask, 
                  const char* callback_contact, int* job_status, 
                  int* failure_code) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_callback_register(job_contact, job_state_mask,
                                                   callback_contact, job_status,
                                                   failure_code);
    Py_END_ALLOW_THREADS
    return ret;
}

int 
callback_unregister(char* job_contact, const char* callback_contact, 
                    int* job_status, int* failure_code) {
    int ret;
    Py_BEGIN_ALLOW_THREADS
    ret = globus_gram_client_job_callback_unregister(job_contact, callback_contact,
                                                     job_status, failure_code);
    Py_END_ALLOW_THREADS
    return ret;
}

PyObject*
client_attr_init() {
    PyObject *tuple, *ptr;
    int ret;
    globus_gram_client_attr_t *attr;
    if (!(tuple=PyTuple_New(2))) {               
        PyErr_SetString(PyExc_RuntimeError, "client_attr_init: PyTuple_New()");
        return NULL;
    }
    attr = (globus_gram_client_attr_t*)globus_malloc(sizeof(globus_gram_client_attr_t));
    ret = globus_gram_client_attr_init(attr);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else {
        ptr = SWIG_NewPointerObj((void*)attr,
                                         SWIGTYPE_p_globus_gram_client_attr_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr);
    }
    return tuple;
}

int
client_attr_destroy(globus_gram_client_attr_t *attr) {
    int ret;
    ret = globus_gram_client_attr_destroy(attr);
    globus_free(attr);
    return ret;
}

PyObject *
client_attr_set_credential(globus_gram_client_attr_t *attr, gss_cred_id_t *cred) {
    int ret;
    PyObject *tuple;
    const char * tmpstr;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, "client_attr_set_credential: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_gram_client_attr_set_credential(*attr, *cred);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    
    return tuple;
}

PyObject*
client_attr_get_credential(globus_gram_client_attr_t *attr) {
    PyObject *tuple, *ptr;
    int ret;
    const char *tmpstr;
    gss_cred_id_t *cred;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, "client_attr_get_credential: PyTuple_New()");
        return NULL;
    }
    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    ret = globus_gram_client_attr_get_credential(*attr, cred);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret != GLOBUS_SUCCESS) {
        tmpstr = globus_gram_client_error_string(ret);
	PyTuple_SET_ITEM(tuple,1,Py_BuildValue("s", tmpstr));
    }
    else {
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 1, ptr); 
    } 
    return tuple;
}

int
client_set_credentials(gss_cred_id_t *cred) {
    return globus_gram_client_set_credentials(*cred);
}

%}        
extern const char*
globus_gram_protocol_error_string(int error_code);

extern void
globus_gram_client_debug(void);

extern int
globus_gram_client_version(void);

