/*############################################################################
# Matt Rodriguez, LBNL
#Copyright (c) 2003, The Regents of the University of California,
#through Lawrence Berkeley National Laboratory 
#(subject to receipt of any required approvals from the U.S. Dept. of Energy). 
#All rights reserved. 
###########################################################################*/
%module gsic
%{
#include "globus_common.h"
#include "globus_gss_assist.h"
#include "gssapi.h"
#include "globus_io.h"
#include "globus_gsi_credential.h"
#include "globus_gsi_proxy.h"
#include "globus_gsi_cert_utils.h"
#include "globus_gsi_system_config.h" 
%}

%include ./src/pyGlobus-typemaps.i

typedef int globus_bool_t;
typedef void* gss_name_t;
typedef void* gss_cred_id_t;
typedef void* gss_ctx_id_t;
typedef int gss_cred_usage_t;
typedef unsigned int gss_qop_t; /* these next two will need to be ifdef'd */
typedef unsigned int OM_uint32; /* for other platforms where they're not uint's */

typedef struct gss_OID_desc_struct {
      OM_uint32 length;
      void       *elements;
} gss_OID_desc,  *gss_OID;

typedef struct gss_OID_set_desc_struct  {
      size_t  count;
      gss_OID elements;
} gss_OID_set_desc,  *gss_OID_set;


typedef struct gss_buffer_desc_struct {
      size_t length;
      void  *value;
} gss_buffer_desc,  *gss_buffer_t;

%constant GSS_EXPORT_CRED_TO_BUFFER=0;
%constant GSS_EXPORT_CRED_EXTERNAL=1;

typedef struct gss_buffer_set_desc_struct {
    size_t                              count;
    gss_buffer_desc *                   elements;
} gss_buffer_set_desc, *gss_buffer_set_t;



%types(gss_OID*);

%constant GSS_C_GSS_CODE=1;
%constant GSS_C_MECH_CODE=2;

%constant GSS_S_COMPLETE=0;
%constant GSS_C_BOTH=0;
%constant GSS_C_INITIATE=1;
%constant GSS_C_ACCEPT=2;

%constant GSS_C_DELEG_FLAG=1;
%constant GSS_C_MUTUAL_FLAG=2;
%constant GSS_C_REPLAY_FLAG=4;
%constant GSS_C_SEQUENCE_FLAG=8;
%constant GSS_C_CONF_FLAG=6;
%constant GSS_C_INTEG_FLAG=32;
%constant GSS_C_ANON_FLAG=64;
%constant GSS_C_PROT_READY_FLAG=128;
%constant GSS_C_TRANS_FLAG=256;

%constant GSS_C_GLOBUS_LIMITED_DELEG_PROXY_FLAG=4096;
%constant GSS_C_GLOBUS_LIMITED_PROXY_FLAG        = 192;
%constant GSS_C_GLOBUS_SSL_COMPATABLE          = 16384;

%constant GSS_C_CALLING_ERROR_OFFSET=24;
%constant GSS_C_ROUTINE_ERROR_OFFSET=16;
%constant GSS_C_SUPPLEMENTARY_OFFSET=0;
%constant GSS_C_CALLING_ERROR_MASK=0377ul;
%constant GSS_C_ROUTINE_ERROR_MASK=0377ul;
%constant GSS_C_SUPPLEMENTARY_MASK=0177777ul;
%constant GSS_S_BAD_MECH=(1ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_NAME=(2ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_NAMETYPE=(3ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_BINDINGS=(4ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_STATUS=(5ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_SIG=(6ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_MIC=GSS_S_BAD_SIG;
%constant GSS_S_NO_CRED=(7ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_NO_CONTEXT=(8ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_DEFECTIVE_TOKEN=(9ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_DEFECTIVE_CREDENTIAL=(10ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_CREDENTIALS_EXPIRED=(11ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_CONTEXT_EXPIRED=(12ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_FAILURE=(13ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_BAD_QOP=(14ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_UNAUTHORIZED=(15ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_UNAVAILABLE=(16ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_DUPLICATE_ELEMENT=(17ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_NAME_NOT_MN=(18ul << GSS_C_ROUTINE_ERROR_OFFSET);
%constant GSS_S_CONTINUE_NEEDED=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 0));
%constant GSS_S_DUPLICATE_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 1));
%constant GSS_S_OLD_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 2));
%constant GSS_S_UNSEQ_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 3));
%constant GSS_S_GAP_TOKEN=(1ul << (GSS_C_SUPPLEMENTARY_OFFSET + 4));

%apply char **OUTPUT { char **charRet };

extern gss_OID GSS_C_NT_USER_NAME;
extern gss_OID GSS_C_NT_MACHINE_UID_NAME;
extern gss_OID GSS_C_NT_STRING_UID_NAME;
extern gss_OID GSS_C_NT_HOSTBASED_SERVICE_X;
extern gss_OID GSS_C_NT_HOSTBASED_SERVICE;
extern gss_OID GSS_C_NT_ANONYMOUS;
extern gss_OID GSS_C_NT_EXPORT_NAME;



%inline %{

#define GLOBUS_GSI_PROXY_GENERIC_POLICY_OID "1.3.6.1.4.1.3536.1.222"
#define GLOBUS_GSI_PROXY_GENERIC_POLICY_SN  "GENERICPOLICY"
#define GLOBUS_GSI_PROXY_GENERIC_POLICY_LN  "Generic Policy Object"


/************************ The following 3 functions are for the grid_proxy_init function **********************************************
*************************************************************************************************************************************
MKR 11/25/02
*************************************************************************************************************************************/

globus_module_descriptor_t* get_module(void) {
    return GLOBUS_GSI_GSS_ASSIST_MODULE;
}

static int
globus_i_gsi_proxy_utils_pwstdin_callback(
    char *                              buf, 
    int                                 num, 
    int                                 w)
{
    int                                 i;

    if (!(fgets(buf, num, stdin))) {
        fprintf(stderr, "Failed to read pass-phrase from stdin\n");
        return -1;
    }
    i = strlen(buf);
    if (buf[i-1] == '\n') {
        buf[i-1] = '\0';
        i--;
    }
    return i;       

}

static void
globus_i_gsi_proxy_utils_key_gen_callback(int p, int n, void * dummy)
{
    char c='B';


    if (p == 0) c='.';
    if (p == 1) c='+';
    if (p == 2) c='*';
    if (p == 3) c='\n';
    fputc(c, stdout);
    fflush(stdout);
}

PyObject *
return_error(
    globus_result_t                     result)
{
    char * error_string = NULL;
    PyObject * tuple;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, "return_error: PyTuple_New()");
        return NULL;
    }
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", result));
    error_string = globus_object_printable_to_string(globus_error_get(result));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("s", error_string));
    return tuple;
}



gss_cred_id_t
get_no_cred_const() {
    return GSS_C_NO_CREDENTIAL;
}

gss_cred_id_t*
get_no_cred_const_ptr() {
    gss_cred_id_t* cred;
    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    *cred = GSS_C_NO_CREDENTIAL;
    return cred;
}

void
free_cred_ptr(gss_cred_id_t* cred)
{
    globus_free(cred);
}
PyObject*
release_cred(gss_cred_id_t* cred) {
    OM_uint32 major_status;
    OM_uint32 minor_status;
    PyObject* tuple;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "release_cred: PyTuple_New()");
        return NULL;
    }
    major_status = gss_release_cred(&minor_status, cred);
    globus_free(cred);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    return tuple;
} 

PyObject *
cred_handle_destroy(globus_gsi_cred_handle_t * cred_handle){
    
    PyObject * tuple;
    char * tmpstr;
    globus_result_t ret;

    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_handle_destroy:: PyTuple_New()");
        return NULL;
    }
    ret = globus_gsi_cred_handle_destroy(*cred_handle);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
    if (ret !=GLOBUS_SUCCESS)
    {
	tmpstr = globus_object_printable_to_string(globus_error_get(ret));
       	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", tmpstr));
    }
    else
    {
	Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    return tuple;
}

PyObject *
create_empty_oid_set()
{
   PyObject * ptr, *tuple;
   OM_uint32 minor_status = 0;
   OM_uint32 major_status = 0;
   gss_OID_set * mechs;
   
   if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_handle_destroy:: PyTuple_New()");
        return NULL;
    }	
   mechs = (gss_OID_set *)globus_malloc(sizeof(gss_OID_set));
   major_status = gss_create_empty_oid_set(&minor_status, mechs);
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
   PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
   if (major_status != GSS_S_COMPLETE)
	{
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
	}
   else
	{
	ptr = SWIG_NewPointerObj((void *)mechs, SWIGTYPE_p_gss_OID_set, 0);
	PyTuple_SET_ITEM(tuple, 2, ptr);
	}	
   return tuple;
}	

PyObject *
new_X509()	
{
    X509_REQ * x509_req;
    PyObject *ptr;
    x509_req =  X509_REQ_new(); 
    
    ptr = SWIG_NewPointerObj((void *)x509_req, SWIGTYPE_p_X509_REQ,0);
    return ptr;	
}

void
free_X509(X509_REQ* req){
    if(req){
        X509_REQ_free(req);
    }
    return;
}

void
free_EVP_PKEY(EVP_PKEY* pkey){
    if(pkey){
        EVP_PKEY_free(pkey);
    }
    
}

PyObject *
new_EVP_PKEY() {
    EVP_PKEY * pkey;
    PyObject *ptr;
    pkey = EVP_PKEY_new(); 
    ptr = SWIG_NewPointerObj((void *)pkey, SWIGTYPE_p_EVP_PKEY,0);
    return ptr;
}


PyObject *
release_oid_set(gss_OID_set * mechs)
{
   PyObject  *tuple;
   OM_uint32 minor_status = 0;
   OM_uint32 major_status = 0;
   
   if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "cred_handle_destroy:: PyTuple_New()");
        return NULL;
    }	

   major_status = gss_release_oid_set(&minor_status, mechs);
   globus_free(mechs);
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
   PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
   
   return tuple;
}	





PyObject *
get_usage_both()
{
  PyObject * ptr;
  gss_cred_usage_t * usage;
  usage = (gss_cred_usage_t *)globus_malloc(sizeof(gss_cred_usage_t));
  *usage = GSS_C_BOTH;
  ptr = SWIG_NewPointerObj((void *)usage, SWIGTYPE_p_gss_cred_usage_t, 0); 
  return ptr;
 
}		

PyObject *
get_usage_initiate()
{
  PyObject * ptr;
  gss_cred_usage_t * usage;
  usage = (gss_cred_usage_t *)globus_malloc(sizeof(gss_cred_usage_t));
  *usage = GSS_C_INITIATE;
  ptr = SWIG_NewPointerObj((void *)usage, SWIGTYPE_p_gss_cred_usage_t, 0); 
  return ptr;
}
PyObject *
get_usage_accept()
{
  PyObject * ptr;
  gss_cred_usage_t * usage;
  usage = (gss_cred_usage_t *)globus_malloc(sizeof(gss_cred_usage_t));
  *usage = GSS_C_ACCEPT;
  ptr = SWIG_NewPointerObj((void *)usage, SWIGTYPE_p_gss_cred_usage_t, 0); 
  return ptr;
}

void 
free_usage(gss_cred_usage_t *usage)
{
  globus_free(usage);
}

PyObject*
acquire_cred(gss_name_t *name, gss_OID_set * desired_mechs,gss_cred_usage_t *cred_usage)
{
	globus_result_t ret;
	OM_uint32 minor_status;	
	OM_uint32 major_status;
	OM_uint32 time_req;
	OM_uint32 time_rec;
	gss_cred_id_t * output_cred;
	gss_OID_set actual_mechs;
	PyObject * ptr;
	PyObject * tuple;
	

	if (!(tuple=PyTuple_New(3)))
    	{
        	PyErr_SetString(PyExc_RuntimeError, 
               	         "acquire_cred:: PyTuple_New()");
        	return NULL;
    	}
	
	output_cred = (gss_cred_id_t *)globus_malloc(sizeof(gss_cred_id_t));	
	major_status = gss_acquire_cred(&minor_status, *name, time_req, *desired_mechs, *cred_usage, output_cred, &actual_mechs, &time_rec);

	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("i", minor_status));
	if (major_status != GSS_S_COMPLETE)
	{
	    Py_INCREF(Py_None);
	    PyTuple_SET_ITEM(tuple, 2, Py_None);
	}
	 else
	 {
	 ptr = SWIG_NewPointerObj((void *)output_cred, SWIGTYPE_p_gss_cred_id_t,0);
	 PyTuple_SET_ITEM(tuple, 2, ptr);
	 }

	return tuple;
}


PyObject*
assist_acquire_cred(gss_cred_usage_t usage, gss_cred_id_t* cred) {
    OM_uint32 major_status;
    OM_uint32 minor_status; 
    PyObject* tuple;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "assist_acquire_cred: PyTuple_New()");
        return NULL;
    }
    major_status = globus_gss_assist_acquire_cred(&minor_status,
						  usage, cred);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    return tuple;
}

/* What other gss_OID for nameTypes can there be??
   Right now I just know of GSS_C_NULL_OID.
   There are about 7 OID types for this 

*/

PyObject*
import_name(PyObject* nameStr, const gss_OID nameType) {
    OM_uint32 major_status;
    OM_uint32 minor_status;
    PyObject *tuple, *ptr;
    gss_buffer_t buf;
    gss_name_t* name;
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "import_name: PyTuple_New()");
        return NULL;
    }
    if (!PyString_Check(nameStr)) {
	PyErr_SetString(PyExc_TypeError, "not a string");
	return NULL;
    }
    buf = (gss_buffer_desc*)globus_malloc(sizeof(gss_buffer_desc));
    name = (gss_name_t *)globus_malloc(sizeof(gss_name_t));	
    buf->length = PyString_Size(nameStr);
    buf->value = PyString_AsString(nameStr);
    major_status = gss_import_name(&minor_status, buf, nameType, name);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    if (major_status != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)name, SWIGTYPE_p_gss_name_t, 0);
	PyTuple_SET_ITEM(tuple, 2, ptr);
    }
    globus_free(buf);
    return tuple;
}

PyObject *
get_default_name()
{
    PyObject * ptr;
    gss_name_t *name;
    name = (gss_name_t *)globus_malloc(sizeof(gss_name_t));
    *name = GSS_C_NO_NAME;
    ptr = SWIG_NewPointerObj((void*)name, SWIGTYPE_p_gss_name_t, 0);
    return ptr;

}	

PyObject*
display_name(gss_name_t* name) {
    PyObject* tuple;
    OM_uint32         major = 0;
    OM_uint32         minor = 0;
    gss_OID           display_oid;
    gss_buffer_t      output_buf;
    char val_buf[1000];
    
	
    if (!(tuple=PyTuple_New(3)))
    {
        PyErr_SetString(PyExc_RuntimeError, "display_name: PyTuple_New()");
        return NULL;
    }
    
    output_buf = (gss_buffer_desc *)globus_malloc(sizeof(gss_buffer_desc));    
    output_buf->value= val_buf;

    major = gss_display_name(&minor, *name, output_buf, &display_oid);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
	PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s#", (char*)output_buf->value, 
						 (int)output_buf->length));
	gss_release_buffer(&minor, output_buf);
    }
    return tuple;
}
 
PyObject*
release_name(gss_name_t* name) {
    PyObject* tuple;
    OM_uint32 major_status;
    OM_uint32 minor_status;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "release_name: PyTuple_New()");
        return NULL;
    }
    major_status = gss_release_name(&minor_status, name);
    globus_free(name);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    return tuple;
}

PyObject*
init_sec_context(gss_ctx_id_t *ctx, const gss_cred_id_t *initiator_cred,
		 gss_name_t *target_name, gss_buffer_t  input_token) {

    PyObject* tuple;
    PyObject* ptr;
    OM_uint32 major_status;
    OM_uint32 minor_status;
    OM_uint32 req_flags = 0;
    OM_uint32 time_req;
    gss_buffer_desc output_token_desc = GSS_C_EMPTY_BUFFER;
    gss_buffer_t output_token = &output_token_desc;
    gss_ctx_id_t test_ctx;
    gss_OID actual_mech;
    OM_uint32 ret_flags;
    OM_uint32 time_rec;
    gss_OID mech_type = NULL;
    gss_ctx_id_t  ctx1;
    if (!(tuple=PyTuple_New(4)))
    {
       PyErr_SetString(PyExc_RuntimeError, 
		       "init_sec_context: PyTuple_New()");
       return NULL;
    }
    //Right now time_req is GSS_C_INDEFINITE, req_flags is NULL, input_channel_bindings are set, mech_type is NULL
    //If you want you can pass actual_mech, ret_flags, time_rec back up
    major_status = gss_init_sec_context(&minor_status, *initiator_cred, ctx,
					*target_name,mech_type, req_flags,
					GSS_C_INDEFINITE, GSS_C_NO_CHANNEL_BINDINGS, 
					input_token, &actual_mech, output_token,
					&ret_flags, &time_rec);
  
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    if (major_status != GSS_S_COMPLETE && major_status != GSS_S_CONTINUE_NEEDED) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    else {
	PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s#", output_token->value,
						 output_token->length));
	ptr = SWIG_NewPointerObj((void *)ctx,SWIGTYPE_p_gss_ctx_id_t, 0);
	PyTuple_SET_ITEM(tuple,3,ptr);
    }
    return tuple;
} 

PyObject *
accept_sec_context(gss_ctx_id_t *ctx, gss_cred_id_t *acceptor_cred_handle, gss_buffer_t input_token){

   PyObject * tuple;
   PyObject * ptr;
   OM_uint32 major_status = 0;
   OM_uint32 minor_status= 0;
   gss_buffer_desc output_token_desc = GSS_C_EMPTY_BUFFER;
   gss_buffer_t output_token = &output_token_desc;
   const gss_channel_bindings_t input_chan_bindings = GSS_C_NO_CHANNEL_BINDINGS;
   /*These are variables that could be passed up to the Python API when major_status is GSS_C_COMPLETE */
   gss_name_t client_name = GSS_C_NO_NAME;
   gss_OID mech_type;
   gss_cred_id_t delegated_cred_handle;
   OM_uint32 ret_flags;
   OM_uint32 time_rec;
   
   if (!(tuple=PyTuple_New(4)))
   {
       PyErr_SetString(PyExc_RuntimeError, 
		       "accept_sec_context: PyTuple_New()");
       return NULL;
   }
  
   major_status = gss_accept_sec_context(&minor_status, ctx,*acceptor_cred_handle,
					 input_token, input_chan_bindings, &client_name,
					 &mech_type, output_token,&ret_flags, 
					 &time_rec,&delegated_cred_handle);
 				  
   PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
   PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
   if (major_status != GSS_S_COMPLETE && major_status != GSS_S_CONTINUE_NEEDED) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    else {
	PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("s#", output_token->value,
						 output_token->length));
	ptr = SWIG_NewPointerObj((void *)ctx,SWIGTYPE_p_gss_ctx_id_t, 0);
	PyTuple_SET_ITEM(tuple,3,ptr);
    }

   return tuple;
}


PyObject*
create_context() {
    PyObject* ptr;
    gss_ctx_id_t *ctx; 
    ctx = (gss_ctx_id_t*)globus_malloc(sizeof(gss_ctx_id_t));
    *ctx = GSS_C_NO_CONTEXT;
    ptr = SWIG_NewPointerObj((void *)ctx, SWIGTYPE_p_gss_ctx_id_t,0);
    return ptr;
}

PyObject*
delete_context(gss_ctx_id_t* ctx) {
    PyObject* tuple;
    OM_uint32 major, minor;
    if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "delete_context: PyTuple_New()");
        return NULL;
    }
    major = gss_delete_sec_context(&minor, ctx, GSS_C_NO_BUFFER);
    globus_free(ctx);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    return tuple;
}
 
PyObject*
inquire_context(gss_ctx_id_t* ctx) {
    PyObject *tuple, *ptr, *ptr2, *ptr3;
    OM_uint32 major, minor, lifetime, ctxFlags;
    int local, open;
    gss_name_t *srcName, *targetName;
    gss_OID* mech;
    if (!(tuple=PyTuple_New(9)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_context: PyTuple_New()");
        return NULL;
    }
    mech = (gss_OID*)globus_malloc(sizeof(gss_OID));
    srcName = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
    targetName = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
    major = gss_inquire_context(&minor, *ctx, srcName, targetName,
				&lifetime, mech, &ctxFlags, &local, &open);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 4, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 5, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 6, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 7, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 8, Py_None);
    }
    else {
	ptr = SWIG_NewPointerObj((void*)srcName, SWIGTYPE_p_gss_name_t, 0);
	PyTuple_SET_ITEM(tuple, 2, ptr);
	ptr2 = SWIG_NewPointerObj((void*)targetName, SWIGTYPE_p_gss_name_t, 0);
	PyTuple_SET_ITEM(tuple, 3, ptr2);
	PyTuple_SET_ITEM(tuple, 4, Py_BuildValue("i", lifetime));
	ptr3 = SWIG_NewPointerObj((void*)mech, SWIGTYPE_p_gss_OID, 0);
	PyTuple_SET_ITEM(tuple, 5, ptr3);
	PyTuple_SET_ITEM(tuple, 6, Py_BuildValue("i", ctxFlags));
	PyTuple_SET_ITEM(tuple, 7, Py_BuildValue("i", local));
	PyTuple_SET_ITEM(tuple, 8, Py_BuildValue("i", open));
    }
    return tuple;
} 



PyObject *
set_buffer(PyObject * bufstring)
{
    PyObject * ptr;
    char * newstr;
    char * tmpstr;
    gss_buffer_t buf;    
    globus_size_t string_size;
    
    buf = (gss_buffer_desc *)globus_malloc(sizeof(gss_buffer_desc));
    string_size = PyString_Size(bufstring);
    newstr = (char *) globus_malloc(sizeof(char) * string_size );
    tmpstr = PyString_AsString(bufstring);
    memcpy(newstr, tmpstr,string_size);
    buf->value = newstr;
    buf->length = string_size;
    ptr = SWIG_NewPointerObj((void*)buf, SWIGTYPE_gss_buffer_t,0);
    return ptr;
}


PyObject *
get_buffer_string(gss_buffer_t  buf)
{
    char * tmpstr;
    PyObject *pystr;    
    pystr = PyString_FromStringAndSize(buf->value, buf->length);
    return pystr;
}

PyObject *
get_empty_buffer()
{
    PyObject *ptr;
    gss_buffer_t  buf;
    buf = (gss_buffer_desc *)globus_malloc(sizeof(gss_buffer_desc));
    buf->value = NULL;
    buf->length = 0;
    ptr = SWIG_NewPointerObj((void *)buf,SWIGTYPE_gss_buffer_t,0);
    return ptr;

}


PyObject * 
free_buffer(gss_buffer_t  buf)
{
    PyObject * tuple;
    OM_uint32 major_status;
    OM_uint32 minor_status;
     if (!(tuple=PyTuple_New(2)))
    {
        PyErr_SetString(PyExc_RuntimeError, 
                        "free_buffer: PyTuple_New()");
        return NULL;
    }
    
    major_status =  gss_release_buffer(&minor_status, buf);
    globus_free(buf);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major_status));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor_status));
    return tuple;
}	

gss_buffer_t
get_no_buffer_const() {
    return GSS_C_NO_BUFFER;
}
// JRB Argument minor_status is not used
PyObject*
display_status(OM_uint32 major_status, OM_uint32 minor_status)
{
    PyObject* list;
    OM_uint32 maj, min, status_type, msgCtx;
    gss_buffer_desc status;
    msgCtx = 0;
    list = PyList_New(0);
    status_type = GSS_C_GSS_CODE;
    do {
	maj = gss_display_status(&min, major_status, status_type,
				 GSS_C_NULL_OID, &msgCtx, &status);
	PyList_Append(list, Py_BuildValue("s#", 
                                         (char*)status.value,
					  status.length));
	gss_release_buffer(&min, &status);
    } while (msgCtx != 0 || 
	(status_type == GSS_C_GSS_CODE && (status_type = GSS_C_MECH_CODE )));
    return list;
}

#ifdef CAS
 
PyObject*
inquire_context_by_oid(gss_ctx_id_t* ctx, PyObject* pyOID) {
    PyObject *tuple, *list;
    OM_uint32 major, minor;
    gss_buffer_set_t set;
    gss_OID_desc oid;
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_context_by_oid: PyTuple_New()");
        return NULL;
    }
    PyString_AsStringAndSize(pyOID,
                             (char **) &(oid.elements),
                             (int *) &(oid.length));
    major = gss_inquire_sec_context_by_oid(&minor, *ctx, &oid, &set);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
        int i;
	list = PyList_New(0);
	for (i=0; i < set.count; i++) {
	    gss_buffer_desc buf = set->elements[i];
	    PyList_Append(list, Py_BuildValue("s#", 
					      (char*)buf.value,
					      buf.length));
	}
	PyTuple_SET_ITEM(tuple, 2, list);
    }
    return tuple;
} 

PyObject*
inquire_cred_by_oid(gss_cred_id_t* cred, PyObject* pyOID) {
    PyObject *tuple, *list;
    OM_uint32 major, minor;
    gss_buffer_set_t set;
    gss_OID_desc oid;
    if (!(tuple=PyTuple_New(3))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_cred_by_oid: PyTuple_New()");
        return NULL;
    }
    PyString_AsStringAndSize(pyOID,
                             (char **) &(oid.elements),
                             (int *) &(oid.length));
    major = gss_inquire_cred_by_oid(&minor, *cred, &oid, &set);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }
    else {
        int i;
	list = PyList_New(0);
	for (i=0; i < set.count; i++) {
	    gss_buffer_desc buf = set->elements[i];
	    PyList_Append(list, Py_BuildValue("s#", 
					      (char*)buf.value,
					      buf.length));
	}
	PyTuple_SET_ITEM(tuple, 2, list);
    }
    return tuple;
}

PyObject*
accept_delegation(gss_ctx_id_t* ctx, PyObject* oids, 
		  PyObject* buffers, OM_uint32 time_req,
		  globus_io_handle_t* io_handle) {
    PyObject *item, *ptr;
    globus_result_t ret;
    OM_uint32 major, minor;
    OM_uint32 time_rec_buf;
    gss_OID_set_desc oidSet;
    gss_buffer_set_desc bufferSet;
    gss_buffer_t buf;
    gss_buffer_desc output_token;
    gss_buffer_desc input_token;
    gss_cred_id_t *cred;
    gss_OID oid, mech;
    globus_size_t read, lenSize, written;
    int oidsLen, i, bufsLen;
    long sendLen;
    long readLen;
    char *globus_error_string = NULL;

    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    *cred = GSS_C_NO_CREDENTIAL;

    lenSize = sizeof(input_token.length);
    oidsLen = PyList_Size(oids);
    mech = NULL;
    if (oidsLen > 0)
    {
        oidSet.count = oidsLen;
        bufsLen = PyList_Size(buffers);
        bufferSet.count = bufsLen;
        oidSet.elements = (gss_OID_desc *)globus_malloc(sizeof(gss_OID_desc) *
                                                        oidsLen);
        for (i=0; i < oidsLen; i++) {
            oid = &(oidSet.elements[i]);
            item = PyList_GetItem(oids, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(oid->elements),
                                     (int *) &(oid->length));
        }

        bufferSet.elements = (gss_buffer_desc*)globus_malloc(
            sizeof(gss_buffer_desc) * bufsLen);
        for (i=0; i < bufsLen; i++) {
            buf = &(bufferSet.elements[i]);
            item = PyList_GetItem(buffers, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(buf->value),
                                     (int *) &(buf->length));
        }
    }
    else
    {
        oidSet.count = 0;
        oidSet.elements = NULL;
        bufferSet.count = 0;
        bufferSet.elements = NULL;
    }
    do {
	Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
        printf("Reading input token length\n");
#endif
	ret = globus_io_read(io_handle, (globus_byte_t*)&readLen,
			     sizeof(readLen), sizeof(readLen), &read);
	Py_END_ALLOW_THREADS
	if (!(ret == GLOBUS_SUCCESS)) {
            globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
            globus_libc_free(globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}
	Py_BEGIN_ALLOW_THREADS
	input_token.length = ntohl(readLen);
#ifdef DEBUG
        printf("Reading %d byte input token\n", input_token.length);
#endif
	input_token.value = globus_malloc(input_token.length);
	ret = globus_io_read(io_handle, (globus_byte_t*)input_token.value,
			     input_token.length, input_token.length, &read);
	Py_END_ALLOW_THREADS
	if (!(ret == GLOBUS_SUCCESS)) {
	    globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
	    globus_libc_free(globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}
	Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
        printf("Calling gss_accept_delegation()\n");
#endif
	major = gss_accept_delegation(&minor, *ctx, 
				      &oidSet, &bufferSet, &input_token,
				      time_req, &time_rec_buf,
				      cred, &mech, &output_token);
	Py_END_ALLOW_THREADS
	if (major != GSS_S_COMPLETE && major != GSS_S_CONTINUE_NEEDED) {
            globus_gss_assist_display_status_str(&globus_error_string,
                                                 "Accepting delegation",
                                                 major, minor, 0);
            gss_release_buffer(&minor, &input_token);
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}
        gss_release_buffer(&minor, &input_token);
#ifdef DEBUG
        printf("gss_accept_delegation() returned %d\n", major);
#endif
        if (output_token.length > 0) 
        {
	    Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
            printf("Writing %d byte token\n", output_token.length);
#endif
            sendLen = htonl(output_token.length);
            ret = globus_io_write(io_handle, (globus_byte_t*)&sendLen,
                                  sizeof(sendLen), &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
	    Py_BEGIN_ALLOW_THREADS
            ret = globus_io_write(io_handle, output_token.value, 
                                  output_token.length, &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
            gss_release_buffer(&minor, &output_token);
        }
    } while (major == GSS_S_CONTINUE_NEEDED);
#ifdef DEBUG
    printf("Returning good credential %x\n", cred);
#endif
    ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
    if (oidsLen > 0) {
	globus_free(oidSet.elements);
    }
    return ptr;
}

PyObject*
init_delegation(gss_ctx_id_t* ctx, gss_cred_id_t* cred, 
		PyObject* oids, PyObject* buffers,
		OM_uint32 time_req,
		globus_io_handle_t* io_handle) {
    PyObject *item;
    globus_result_t ret;
    OM_uint32 major, minor;
    gss_OID_set_desc oidSet;
    gss_buffer_set_desc bufferSet;
    gss_buffer_t buf;
    gss_buffer_desc output_token;
    gss_buffer_t input_token = GSS_C_NO_BUFFER;
    gss_buffer_desc input_token_buf;
    gss_OID oid;
    gss_OID mech = NULL;
    globus_size_t lenSize;
    globus_size_t written;
    globus_size_t read;
    int oidsLen, i, bufsLen;
    long sendLen;
    long readLen;
    char *globus_error_string;
    
    oidsLen = PyList_Size(oids);

    if (oidsLen > 0) 
    {
        oidSet.count = oidsLen;
        bufsLen = PyList_Size(buffers);
        bufferSet.count = bufsLen;
        oidSet.elements = (gss_OID_desc*)globus_malloc(sizeof(gss_OID_desc) *
                                                       oidsLen);
        for (i=0; i < oidsLen; i++) {
            oid = &(oidSet.elements[i]);
            item = PyList_GetItem(oids, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(oid->elements),
                                     (int *) &(oid->length));
        }

        bufferSet.elements = (gss_buffer_desc*)globus_malloc(
            sizeof(gss_buffer_desc) * bufsLen);
        for (i=0; i < bufsLen; i++) {
            buf = &(bufferSet.elements[i]);
            item = PyList_GetItem(buffers, i);
            PyString_AsStringAndSize(item,
                                     (char **) &(buf->value),
                                     (int *) &(buf->length));
        }
    }
    else
    {
        oidSet.count = 0;
        oidSet.elements = NULL;
        bufferSet.count = 0;
        bufferSet.elements = NULL;
    }
    do {
	Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
        printf("DEBUG: Calling gss_init_delegation\n");
#endif
	major = gss_init_delegation(&minor, *ctx, *cred, mech,
				    &oidSet, &bufferSet, 
				    input_token, time_req, &output_token);
#ifdef DEBUG
        printf("DEBUG: gss_init_delegation returned %d\n", major);
#endif
	Py_END_ALLOW_THREADS
	if (major != GSS_S_COMPLETE && major != GSS_S_CONTINUE_NEEDED) {
            globus_gss_assist_display_status_str(&globus_error_string,
                                                 "Initing delegation",
                                                 major, minor, 0);
            gss_release_buffer(&minor, input_token);
	    PyErr_SetString(PyExc_RuntimeError, globus_error_string);
	    if (oidsLen > 0) {
		globus_free(oidSet.elements);
	    }
	    return NULL;
	}

        /* This call is ok if input_token == GSS_C_NO_BUFFER */
        gss_release_buffer(&minor, input_token);

        if (output_token.length > 0)
        {
	    Py_BEGIN_ALLOW_THREADS
#ifdef DEBUG
            printf("Writing %d byte token\n", output_token.length);
#endif
            sendLen = htonl(output_token.length);
            ret = globus_io_write(io_handle, (globus_byte_t*)&sendLen,
                                  sizeof(sendLen), &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
	    Py_BEGIN_ALLOW_THREADS
            ret = globus_io_write(io_handle, output_token.value, 
                                  output_token.length, &written);
	    Py_END_ALLOW_THREADS
            if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
            }
            gss_release_buffer(&minor, &output_token);
        }
        
	if (major == GSS_S_CONTINUE_NEEDED) {
	    Py_BEGIN_ALLOW_THREADS
	    ret = globus_io_read(io_handle, (globus_byte_t*)&readLen,
				 sizeof(readLen), sizeof(readLen), &read);
	    Py_END_ALLOW_THREADS
	    if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
	    }
	    Py_BEGIN_ALLOW_THREADS
	    input_token_buf.length = ntohl(readLen);
#ifdef DEBUG
            printf("Reading %d byte input token\n", input_token_buf.length);
#endif
	    input_token_buf.value = globus_malloc(input_token_buf.length);  
	    ret = globus_io_read(io_handle,
                                 (globus_byte_t*)input_token_buf.value,
				 input_token_buf.length,
                                 input_token_buf.length,
                                 &read);
	    Py_END_ALLOW_THREADS
	    if (!(ret == GLOBUS_SUCCESS)) {
                globus_error_string = globus_object_printable_to_string(globus_error_get(ret));
                PyErr_SetString(PyExc_RuntimeError, globus_error_string);
                globus_libc_free(globus_error_string);
		if (oidsLen > 0) {
		    globus_free(oidSet.elements);
		}
                return NULL;
	    }
            input_token = &input_token_buf;
	}
    } while (major == GSS_S_CONTINUE_NEEDED);
    Py_INCREF(Py_None);
    return Py_None;
}

#endif /* CAS */

PyObject*
export_cred(gss_cred_id_t *cred,
            int option_req)
{
    OM_uint32 major;
    OM_uint32 minor;
    gss_OID p_desired_mech = NULL;
    gss_buffer_desc export_buffer;
    char *globus_error_string = NULL;
    PyObject* tuple;

#ifdef DEBUG
    printf("Exporting cred %x (%x)\n", cred, *cred);
#endif

    if (!(tuple=PyTuple_New(3))) {
	PyErr_SetString(PyExc_RuntimeError, 
			"export_cred: PyTuple_New()");
	return NULL;
    }
    major = gss_export_cred(&minor,
                            *cred,
                            p_desired_mech,
                            option_req,
                            &export_buffer);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));

#ifdef DEBUG
    printf("Export done. major = %d\n", major);
#endif
    if (major == GLOBUS_SUCCESS)
    {
        PyTuple_SET_ITEM(tuple, 2,
                         Py_BuildValue("s#", (char *) export_buffer.value,
                                       export_buffer.length));
        gss_release_buffer(&minor, &export_buffer);
    }
    else
    {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
    }

    return tuple;
}
                
PyObject*
import_cred(int option_req, PyObject* credHandle, int time)
{    
    OM_uint32 major;
    OM_uint32 minor;
    OM_uint32 timeRec;
    gss_OID p_desired_mech = NULL;
    gss_buffer_t buf;
    char *globus_error_string = NULL;
    PyObject* tuple;
    gss_cred_id_t* cred;
    PyObject* ptr;
    if (!(tuple=PyTuple_New(4))) {
	PyErr_SetString(PyExc_RuntimeError, 
			"import_cred: PyTuple_New()");
	return NULL;
    }
    cred = (gss_cred_id_t*)globus_malloc(sizeof(gss_cred_id_t));
    buf = (gss_buffer_desc*)globus_malloc(sizeof(gss_buffer_desc));
    buf->length = PyString_Size(credHandle);
    buf->value = PyString_AsString(credHandle);
    major = gss_import_cred(&minor,
                            cred,
                            p_desired_mech,
                            option_req,
                            buf,
                            (OM_uint32)time,
                            &timeRec);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major == GLOBUS_SUCCESS)
    {
        ptr = SWIG_NewPointerObj((void*)cred, SWIGTYPE_p_gss_cred_id_t, 0);
        PyTuple_SET_ITEM(tuple, 2, ptr);
        PyTuple_SET_ITEM(tuple, 3, Py_BuildValue("i", timeRec));
    }
    else
    {
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 2, Py_None);
        Py_INCREF(Py_None);
        PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    globus_free(buf);
    return tuple;
}

PyObject*
inquire_cred(gss_cred_id_t* cred) {
    PyObject *tuple, *list, *name_p, life_p;
    OM_uint32 major;
    OM_uint32 minor;
    gss_name_t name;
    OM_uint32 life;
    gss_cred_usage_t cred_usage;
    gss_OID_set output_mechs;
    if (!(tuple=PyTuple_New(4))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "inquire_cred: PyTuple_New()");
        return NULL;
    }
    name = (gss_name_t*)globus_malloc(sizeof(gss_name_t));
    major = gss_inquire_cred(&minor, *cred, name, &life, 
                              &cred_usage, &output_mechs);

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", major));
    PyTuple_SET_ITEM(tuple, 1, Py_BuildValue("i", minor));
    if (major != GSS_S_COMPLETE) {
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 2, Py_None);
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 3, Py_None);
    }
    else {
      
	PyTuple_SET_ITEM(tuple, 2, Py_BuildValue("i", life));
	name_p = SWIG_NewPointerObj((void*)name, SWIGTYPE_p_gss_name_t, 0);
	Py_INCREF(name_p);
	PyTuple_SET_ITEM(tuple, 3, name_p);

    }
    return tuple;
}


PyObject *
grid_proxy_init(int debug)
{
    globus_result_t                     result  = GLOBUS_SUCCESS;
    /* default proxy to 512 bits */
    int                                 key_bits    = 512;
    /* default to a 12 hour cert */
    int                                 valid       = 12*60;
    /* dont restrict the proxy */
    int                                 verify      = 0;
    char *                              user_cert_filename = NULL;
    char *                              user_key_filename = NULL;
    char *                              tmp_user_cert_filename = NULL;
    char *                              tmp_user_key_filename = NULL;
    char *                              proxy_out_filename = NULL;
    char *                              ca_cert_dir = NULL;
    char *                              program = NULL;
    globus_gsi_proxy_handle_t           proxy_handle = NULL;
    globus_gsi_proxy_handle_attrs_t     proxy_handle_attrs = NULL;
    globus_gsi_callback_data_t          callback_data = NULL;
    globus_gsi_cred_handle_attrs_t      cred_handle_attrs = NULL;
    globus_gsi_cred_handle_t            cred_handle = NULL;
    globus_gsi_cred_handle_t            proxy_cred_handle = NULL;
    globus_gsi_cert_utils_proxy_type_t  proxy_type = GLOBUS_FULL_PROXY;
    BIO *                               pem_proxy_bio = NULL;
    time_t                              goodtill;
    time_t                              lifetime;
    char *                              restriction_buf = NULL;
    size_t                              restriction_buf_len = 0;
    char *                              restriction_filename = NULL;
    char *                              policy_language = NULL;
    int                                 policy_NID;
    int                                 (*pw_cb)() = NULL;
    char *                              trusted_subgroup = NULL;
    char *                              untrusted_subgroup = NULL;
    char *                              subgroup = NULL;
    int                                 return_value = 0;
    globus_gsi_statcheck_t          file_status;
    char *                          proxy_absolute_path = NULL;
    char *                          temp_filename = NULL;
    char *                          temp_dir = NULL;	
    PyObject * tuple;
    PyObject * errTuple;
    PyObject * ptr;
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "grid_proxy_init: PyTuple_New()");
        return NULL;
    }
    
    if(globus_module_activate(GLOBUS_GSI_PROXY_MODULE) != (int)GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't load module: GLOBUS_GSI_PROXY_MODULE.\n"
            "Make sure Globus is installed correctly.\n\n");
        exit(1);
    }

    if(globus_module_activate(GLOBUS_GSI_CALLBACK_MODULE) != (int)GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't load module: GLOBUS_GSI_CALLBACK_MODULE.\n"
            "Make sure Globus is installed correctly.\n\n");
        exit(1);
    }

   
    result = globus_gsi_proxy_handle_attrs_init(
        &proxy_handle_attrs);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr, 
                            "\n\nERROR: Couldn't initialize "
                            "the proxy handle attributes.\n");
	return_error(result);
        
    }

    /* set the time valid in the proxy handle attributes
     * used to be hours - now the time valid needs to be set in minutes 
     */
    result = globus_gsi_proxy_handle_attrs_set_time_valid(
        proxy_handle_attrs, valid);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                            "\n\nERROR: Couldn't set the validity time "
                            "of the proxy cert to %d minutes.\n", valid);
	
        errTuple = return_error(result);
	return errTuple;
    }

    /* set the key bits for the proxy cert in the proxy handle
     * attributes
     */
    result = globus_gsi_proxy_handle_attrs_set_keybits(
        proxy_handle_attrs, key_bits);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                            "\n\nERROR: Couldn't set the key bits for "
                            "the private key of the proxy certificate\n");
	errTuple = return_error(result);
	return errTuple;  
    }
    
    result = globus_gsi_proxy_handle_attrs_set_key_gen_callback(
        proxy_handle_attrs, 
        globus_i_gsi_proxy_utils_key_gen_callback);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't set the key generation callback function\n");
        errTuple = return_error(result);
	return errTuple;
    }

    result = globus_gsi_proxy_handle_init(&proxy_handle, proxy_handle_attrs);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't initialize the proxy handle\n");
        errTuple = return_error(result);
	return errTuple;
    }

    if(!user_cert_filename || !user_key_filename)
    {
        result = GLOBUS_GSI_SYSCONFIG_GET_USER_CERT_FILENAME(
            &tmp_user_cert_filename,
            &tmp_user_key_filename);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(stderr,
                                "\n\nERROR: Couldn't find valid credentials "
                                "to generate a proxy.\n");
	    errTuple = return_error(result);
	    return errTuple;
        }

        if(tmp_user_cert_filename == tmp_user_key_filename)
        {
            /* supposed to be a pkcs12 formated credential */
            user_cert_filename = user_key_filename
                               = tmp_user_key_filename;
        }
    }

    if(!user_cert_filename)
    {
        user_cert_filename = tmp_user_cert_filename;
    }

    if(!user_key_filename)
    {
        user_key_filename = tmp_user_key_filename;
    }
    
    if(debug)
    {
        globus_libc_fprintf(stderr,
                            "\nUser Cert File: %s\nUser Key File: %s\n",
                            user_cert_filename, user_key_filename);
    }

    if (!strncmp(user_cert_filename, "SC:", 3))
    {
        EVP_set_pw_prompt("Enter card pin:");
    }
    else
    {
        EVP_set_pw_prompt(debug? "Enter GRID pass phrase:" :
                          "Enter GRID pass phrase for this identity:");
    }

    if(!ca_cert_dir)
    {
        result = GLOBUS_GSI_SYSCONFIG_GET_CERT_DIR(&ca_cert_dir);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't find a valid trusted certificate "
                "directory\n");
	    errTuple = return_error(result);
	    return errTuple;
            
        }
    }
   
      if(debug)
    {
        globus_libc_fprintf(stderr, 
                            "\nTrusted CA Cert Dir: %s\n", 
                            ca_cert_dir);
    }

    if(!proxy_out_filename)
    {
        result = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME(
            &proxy_out_filename,
            GLOBUS_PROXY_FILE_OUTPUT);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't find a valid location "
                "to write the proxy file\n");
            
	    errTuple = return_error(result);
	    return errTuple;
        }
    }
    else
    {
        /* verify that the directory path of proxy_out_filename
         * exists and is writeable
         */
        
       globus_gsi_statcheck_t          file_status;
       char *                          proxy_absolute_path = NULL;
       char *                          temp_filename = NULL;
       char *                          temp_dir = NULL;	
	
        /* first, make absolute path */
        result = GLOBUS_GSI_SYSCONFIG_MAKE_ABSOLUTE_PATH_FOR_FILENAME(
            proxy_out_filename,
            &proxy_absolute_path);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Can't create the absolute path "
                "of the proxy filename: %s",
                proxy_out_filename);
            errTuple = return_error(result);
	    return errTuple;;
        }

        if(proxy_out_filename)
        {
            free(proxy_out_filename);
        }
        
        proxy_out_filename = proxy_absolute_path;

        /* then split */
        result = GLOBUS_GSI_SYSCONFIG_SPLIT_DIR_AND_FILENAME(
            proxy_absolute_path,
            &temp_dir,
            &temp_filename);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Can't split the full path into "
                "directory and filename. The full path is: %s", 
                proxy_absolute_path);
            if(proxy_absolute_path)
            {
                free(proxy_absolute_path);
                proxy_absolute_path = NULL;
            }
            errTuple = return_error(result);
	    return errTuple;;
        }
                
        result = GLOBUS_GSI_SYSCONFIG_FILE_EXISTS(temp_dir, &file_status);
        if(result != GLOBUS_SUCCESS ||
           file_status != GLOBUS_FILE_DIR)
        {
            if(temp_dir)
            {
                free(temp_dir);
                temp_dir = NULL;
            }
            
            if(temp_filename)
            {
                free(temp_filename);
                temp_filename = NULL;
            }

            globus_libc_fprintf(
                stderr, 
                "\n\nERROR: %s is not a valid directory for writing the "
                "proxy certificate\n\n",
                temp_dir);
            if(result != GLOBUS_SUCCESS)
            {
                errTuple = return_error(result);
	    	return errTuple;;
            }
            else
            {
                exit(1);
            }
        }

        if(temp_dir);
        {
            free(temp_dir);
            temp_dir = NULL;
        }
        
        if(temp_filename)
        {
            free(temp_filename);
            temp_filename = NULL;
        }
    }

    if(debug)
    {
        globus_libc_fprintf(stderr, "\nOutput File: %s\n", proxy_out_filename);
    }


    result = globus_gsi_cred_handle_attrs_init(&cred_handle_attrs);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                            "\n\nERROR: Couldn't initialize credential "
                            "handle attributes\n");
        
	errTuple = return_error(result);
	return errTuple;
    }

    result = globus_gsi_cred_handle_attrs_set_ca_cert_dir(
        cred_handle_attrs, 
        ca_cert_dir);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't set the trusted CA certificate "
            "directory in the credential handle attributes\n");
        
	errTuple = return_error(result);
	return errTuple;
    }

    result = globus_gsi_cred_handle_init(&cred_handle, cred_handle_attrs);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                            "\n\nERROR: Couldn't initialize credential "
                            "handle\n");
        
	errTuple = return_error(result);
	return errTuple;
    }

    result = globus_gsi_cred_handle_attrs_destroy(cred_handle_attrs);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                            "\n\nERROR: Couldn't destroy credential "
                            "handle attributes.\n");
        
	errTuple = return_error(result);
	return errTuple;
    }

    if(strstr(user_cert_filename, ".p12"))
    {
        /* we have a pkcs12 credential */
      printf("Found pkcs13 credential\n");  
      result = globus_gsi_cred_read_pkcs12(
            cred_handle,
            user_cert_filename);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't read in PKCS12 credential "
                "from file: %s\n", user_cert_filename);
            
	    errTuple = return_error(result);
	    return errTuple;
	}

        if (!debug)
        {
            char *                          subject = NULL;
            result = globus_gsi_cred_get_subject_name(cred_handle, &subject);
            if(result != GLOBUS_SUCCESS)
            {
                globus_libc_fprintf(
                    stderr,
                    "\n\nERROR: The subject name of the "
                    "user certificate could "
                    "not be retrieved\n");
                
		errTuple = return_error(result);
		return errTuple;
            }
            
            printf("Your identity: %s\n", subject);
            if(subject)
            {
                free(subject);
                subject = NULL;
            }
        }
    }
    else if(user_cert_filename != user_key_filename)
    {
  
        result = globus_gsi_cred_read_cert(
            cred_handle,
            user_cert_filename);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't read user certificate\n"
                "cert file location: %s\n\n", 
                user_cert_filename);
            
	    errTuple = return_error(result);
	    return errTuple;
        }

        if (debug)
        {
            char *                          subject = NULL;
            result = globus_gsi_cred_get_subject_name(cred_handle, &subject);
            if(result != GLOBUS_SUCCESS)
            {
                globus_libc_fprintf(
                    stderr,
                    "\n\nERROR: The subject name of the "
                    "user certificate could "
                    "not be retrieved\n");
                
		errTuple = return_error(result);
		return errTuple;
	    }
            
            printf("Your identity: %s\n", subject);
            if(subject)
            {
                free(subject);
                subject = NULL;
            }
        }
        
	printf("Waiting for passphrase\n");
        result = globus_gsi_cred_read_key(
            cred_handle,
            user_key_filename,
            pw_cb);
        printf("Passphrase Entered\n");  
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't read user key\n"
                "key file location: %s\n\n", 
                user_key_filename);
            errTuple = return_error(result);
	    return errTuple;
        }
    }
    else
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: The user certificate filename: %s\n"
            "and key filename: %s\nare the same, but they do not point to a "
            "PKCS12 formatted (.p12 extension) certificate\n\n",
            user_cert_filename, user_key_filename);
        exit(1);
    }

   
    /* add restrictions now */
    if(restriction_filename)
    {
        int                             restriction_buf_size = 0;
        FILE *                          restriction_fp = NULL;
        
	printf("Looking at restrictions\n");
        restriction_fp = fopen(restriction_filename, "r");
        if(!restriction_fp)
        {
            fprintf(stderr, 
                    "\n\nERROR: Unable to open restrictions "
                    " file: %s\n\n", restriction_filename);
            exit(1);
        }

        do 
        {
            restriction_buf_size += 512;
            
            /* First time through this is a essentially a malloc() */
            restriction_buf = realloc(restriction_buf,
                                      restriction_buf_size);

            if (restriction_buf == NULL)
            {
                fprintf(stderr, 
                        "\nAllocation of space for "
                        "restriction buffer failed\n\n");
                exit(1);
            }

            restriction_buf_len += 
                fread(&restriction_buf[restriction_buf_len], 1, 
                      512, restriction_fp);

            /*
             * If we read 512 bytes then restriction_buf_len and
             * restriction_buf_size will be equal and there is
             * probably more to read. Even if there isn't more
             * to read, no harm is done, we just allocate 512
             * bytes we don't end up using.
             */
        }
        while (restriction_buf_len == restriction_buf_size);
        
        if (restriction_buf_len > 0)
        {
            if(!policy_language)
            {
                policy_NID = 
                    OBJ_create(GLOBUS_GSI_PROXY_GENERIC_POLICY_OID,
                               GLOBUS_GSI_PROXY_GENERIC_POLICY_SN,
                               GLOBUS_GSI_PROXY_GENERIC_POLICY_LN);
            }
            else
            {
                policy_NID = 
                    OBJ_create(policy_language,
                               policy_language,
                               policy_language);
            }

            result = globus_gsi_proxy_handle_set_policy(
                proxy_handle,
                restriction_buf,
                restriction_buf_len,
                policy_NID);
            if(result != GLOBUS_SUCCESS)
            {
                globus_libc_fprintf(stderr,
                                    "\n\nERROR: Can't set the restriction "
                                    "policy in the proxy handle\n");
               errTuple = return_error(result);
	       return errTuple; 
            }
        }   

        fclose(restriction_fp);
    }

    subgroup = trusted_subgroup ? trusted_subgroup : untrusted_subgroup;

    if(subgroup)
    {
        result = globus_gsi_proxy_handle_set_group(
            proxy_handle,
            subgroup,
            trusted_subgroup ? 1 : 0);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(stderr,
                                "\n\nERROR: Can't set the group of the proxy "
                                "credential\n");
            
	    errTuple = return_error(result);
	    return errTuple;
	}
    }

    if (debug)
    {
        printf("Creating proxy ");
        fflush(stdout);
    }

    result = globus_gsi_proxy_create_signed(
        proxy_handle,
        cred_handle,
        &proxy_cred_handle);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                           "\n\nERROR: Couldn't create proxy certificate\n");
        
	errTuple = return_error(result);
	return errTuple;
    }

    if (debug)
    {
        fprintf(stdout, " Done\n");
    }

    if(verify)
    {
        result = globus_gsi_callback_data_init(&callback_data);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(stderr,
                                "\n\nERROR: Couldn't initialize callback data "
                                "for credential verification\n");
            
	    errTuple = return_error(result);
	    return errTuple;
	}

        result = globus_gsi_callback_set_cert_dir(
            callback_data,
            ca_cert_dir);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(stderr,
                                "\n\nERROR: Couldn't set the trusted "
                                "certificate directory in the callback "
                                "data\n");
            
	    errTuple = return_error(result);
	    return errTuple;
        }

        result = globus_gsi_cred_verify_cert_chain(
           proxy_cred_handle,
            callback_data);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't verify the authenticity of the user's "
                "credential to generate a proxy from.\n");
            
	    errTuple = return_error(result);
	    return errTuple;
	}

        globus_libc_fprintf(
            stdout,
            "Proxy Verify OK\n");
    }

    if(ca_cert_dir)
    {
        free(ca_cert_dir);
        ca_cert_dir = NULL;
    }

    result = globus_gsi_cred_write_proxy(proxy_cred_handle,
                                         proxy_out_filename);	

    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: The proxy credential could not be "
            "written to the output file.\n");
        
	errTuple = return_error(result);
	return errTuple;
    }

    if(proxy_out_filename)
    {
        free(proxy_out_filename);
        proxy_out_filename = NULL;
    }

    result = globus_gsi_cred_get_lifetime(
        cred_handle,
        &lifetime);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                            "\n\nERROR: Can't get the lifetime of the proxy "
                            "credential.\n");
        
	errTuple = return_error(result);
	    return errTuple;
    }

    result = globus_gsi_cred_get_goodtill(
        proxy_cred_handle,
        &goodtill);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(stderr,
                            "\n\nERROR: Can't get the expiration date of the "
                            "proxy credential.\n");
         
    }

    if(lifetime < 0)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Your certificate has expired: %s\n\n", 
            asctime(localtime(&goodtill)));
        exit(2);
    }
    else if(lifetime < (valid * 60))
    {
        globus_libc_fprintf(
            stderr, 
            "\nWarning: your certificate and proxy will expire %s "
            "which is within the requested lifetime of the proxy\n",
            asctime(localtime(&goodtill)));
        return_value = 1;
    }
    else if(!debug)
    {
        globus_libc_fprintf(
            stdout,
            "Your proxy is valid until: %s", 
            asctime(localtime(&goodtill)));
    }

    BIO_free(pem_proxy_bio);

    globus_gsi_proxy_handle_destroy(proxy_handle);
    globus_gsi_cred_handle_destroy(cred_handle);
    globus_gsi_callback_data_destroy(callback_data); 
    globus_gsi_cred_handle_destroy(proxy_cred_handle);
    if(tmp_user_cert_filename)
    {
        free(tmp_user_cert_filename);
    }

    if(tmp_user_key_filename)
    {
        free(tmp_user_key_filename);
    }

    globus_module_deactivate(GLOBUS_GSI_PROXY_MODULE);
    globus_module_deactivate(GLOBUS_GSI_CALLBACK_MODULE);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
    
    Py_INCREF(Py_None); 
    PyTuple_SET_ITEM(tuple, 1, Py_None);
    return tuple;

   
}

PyObject * 
grid_proxy_info(int verbose)
{
    int                                 strength          = 0;
    int                                 bits              = 0;
    int                                 time_valid        = 0;
    int                                 exists_flag       = 0;
    int                                 time_valid_flag   = 0;
    int                                 hours_flag        = 0;
    int                                 bits_flag         = 0;
    int                                 is_valid          = 0;
    
    char *                              proxy_filename = NULL;
    globus_gsi_statcheck_t              status;
    char *                              subject;
    char *                              issuer;
    globus_gsi_cert_utils_proxy_type_t  proxy_type;
    char *                              proxy_type_name;
    time_t                              lifetime;
    globus_gsi_cred_handle_t            proxy_cred = NULL;
    X509 *                              proxy_cert = NULL;
    EVP_PKEY *                          proxy_pubkey = NULL;
    globus_result_t                     result;

    char proxy_string[500];
    PyObject * tuple;
    PyObject * errTuple;
    PyObject * ptr;
    PyObject * py_proxy_string;
    PyObject * py_tmp_string;

    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "grid_proxy_info: PyTuple_New()");
        return NULL;
    }
    if(globus_module_activate(GLOBUS_GSI_PROXY_MODULE) != (int)GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't load module: GLOBUS_GSI_PROXY_MODULE.\n"
            "Make sure Globus is installed correctly.\n\n");
        exit(1);
    }
   
    if(proxy_filename)
    {
        result = GLOBUS_GSI_SYSCONFIG_CHECK_KEYFILE(proxy_filename,
                                                    &status);

        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't find a valid proxy.\n");
            errTuple = return_error(result);
	    return errTuple;
        }

        if(status != GLOBUS_FILE_VALID)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't find a valid proxy.\n");
            exit(1);
        }
    }
    else
    { 
        result = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME(
            &proxy_filename,
            GLOBUS_PROXY_FILE_INPUT);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\n\nERROR: Couldn't find a valid proxy.\n");
            errTuple = return_error(result);
	    return errTuple;
        }
    }

    result = globus_gsi_cred_handle_init(&proxy_cred, NULL);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't initialize proxy credential handle\n");
        errTuple = return_error(result);
	return errTuple;
    }
    
    result = globus_gsi_cred_read_proxy(proxy_cred, proxy_filename);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't read proxy from: %s\n", proxy_filename);
        errTuple = return_error(result);
	return errTuple;
    }

    result = globus_gsi_cred_get_cert(proxy_cred, &proxy_cert);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get the proxy certificate from "
            "the proxy credential.\n");
        errTuple = return_error(result);
	return errTuple;
    }

    if ((proxy_pubkey = X509_get_pubkey(proxy_cert)) == NULL)
    {
	globus_libc_fprintf(
            stderr,
            "\n\nERROR: unable to load public key from proxy\n");
        exit(1);
    }

    /* The things we will need to know below: subject, issuer,
       strength, validity, type */

    /* subject */
    result = globus_gsi_cred_get_subject_name(proxy_cred, &subject);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get a valid subject "
            "name from the proxy credential.\n");
        errTuple = return_error(result);
	return errTuple;
    }

    /* issuer */
    issuer = X509_NAME_oneline(X509_get_issuer_name(proxy_cert), NULL, 0);

    /* validity: set time_diff to time to expiration (in seconds) */
    result = globus_gsi_cred_get_lifetime(proxy_cred,
                                          &lifetime);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get a valid lifetime "
            "for the proxy credential.\n");
        errTuple = return_error(result);
	return errTuple;
    }

    /* strength: set strength to key size (in bits) */
    strength = 8 * EVP_PKEY_size(proxy_pubkey);

    /* check if proxy is valid in our own defined sense */
    if (exists_flag)
    {
	is_valid = (lifetime >= (time_valid * 60)) && 
                   (strength >= bits) ? 0 : 1;
    }
    else
    {
	is_valid = 0;
    }

    /* type: restricted, limited or full */
    result = globus_gsi_cred_check_proxy(proxy_cred,
                                         &proxy_type);
    if(result != GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't get the proxy type "
            "from the proxy credential\n");
        errTuple = return_error(result);
	return errTuple;
    }
    
    switch(proxy_type)
    {
    case GLOBUS_FULL_PROXY:
        proxy_type_name = "full";
        break;
    case GLOBUS_LIMITED_PROXY:
        proxy_type_name = "limited";
        break;
    case GLOBUS_RESTRICTED_PROXY:
        proxy_type_name = "restricted";
        break;
    case GLOBUS_NOT_PROXY:
        proxy_type_name = "not a proxy";
        break;
    case GLOBUS_ERROR_PROXY:
    default:

        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Invalid proxy type\n\n");
        exit(1);
    }
    
    
      
     sprintf(proxy_string, "subject  : %s\n" 
		   "issuer   : %s\n" 
		   "type     : %s\n" 
		   "strength : %d bits\n"
                   "path     : %s\n"
		   "timeleft : ",
		   subject,
		   issuer,
		   proxy_type_name,
		   strength,
                   proxy_filename);
    
     py_proxy_string = PyString_FromString(proxy_string);
   
	 
     if (lifetime <= 0)
	    lifetime = 0;

      sprintf(proxy_string, "%ld:%02ld:%02ld\n",
		   (long)(lifetime / 3600),
		   (long)(lifetime % 3600) / 60,
		   (long)lifetime % 60 );
      
      py_tmp_string = PyString_FromString(proxy_string);
      
      PyString_Concat(&py_proxy_string, py_tmp_string);
	    if (lifetime > 3600 * 24)
	    {
		sprintf(proxy_string, "  (%.1f days)\n", (float)(lifetime / 3600) / 24.0);
		py_tmp_string = PyString_FromString(proxy_string);
		PyString_Concat(&py_proxy_string, py_tmp_string);
		
	    }
   
    free(subject);
    free(issuer);

    globus_module_deactivate(GLOBUS_GSI_PROXY_MODULE);
    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
    PyTuple_SET_ITEM(tuple, 1, py_proxy_string);
    return tuple;
}

static int
globus_i_gsi_proxy_utils_clear_and_remove(
    char *                              filename,
    int                                 flag) 
{
    int                                 f;
    int                                 rec;
    int                                 left;
    long                                size;
    char                                msg[65] 
        = "Destroyed by globus_proxy_destroy\r\n";

    if (flag)
        fprintf(stderr, "Would remove %s\n", filename ? filename : "(null)");
    else
    {
        f = open(filename, O_RDWR);
        if (f) 
        {
            size = lseek(f, 0L, SEEK_END);
            lseek(f, 0L, SEEK_SET);
            if (size > 0) 
            {
                rec = size / 64;
                left = size - rec * 64;
                while (rec)
                {
                    write(f, msg, 64);
                    rec--;
                }
                if (left) 
                    write(f, msg, left);
            }
            close(f);
        }
        remove(filename);
    }
    return 0;
}

PyObject *
grid_proxy_destroy()
{
    char *                              default_full_file = NULL;
    globus_result_t                     result = GLOBUS_SUCCESS;
    PyObject *                          tuple;
    PyObject *                          errTuple;

    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "grid_proxy_destroy:: PyTuple_New()");
        return NULL;
    }
    
    if(globus_module_activate(GLOBUS_GSI_PROXY_MODULE) != (int)GLOBUS_SUCCESS)
    {
        globus_libc_fprintf(
            stderr,
            "\n\nERROR: Couldn't load module: GLOBUS_GSI_PROXY_MODULE.\n"
            "Make sure Globus is installed correctly.\n\n");
	exit(1);
    }

    result = GLOBUS_GSI_SYSCONFIG_GET_PROXY_FILENAME(&default_full_file,
                                                         GLOBUS_PROXY_FILE_INPUT);
        if(result != GLOBUS_SUCCESS)
        {
            globus_libc_fprintf(
                stderr,
                "\nERROR: Proxy file doesn't exist or has bad permissions\n");
	     errTuple = return_error(result);
	     return errTuple;
        }



    globus_i_gsi_proxy_utils_clear_and_remove(default_full_file, 0);
 done:

    if(default_full_file)
    { 
        free(default_full_file);
    }

    globus_module_deactivate(GLOBUS_GSI_PROXY_MODULE);
    

    PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
    Py_INCREF(Py_None);
    PyTuple_SET_ITEM(tuple, 1, Py_None);
    return tuple;
    
}

void
globus_i_gsi_proxy_utils_print_error(
    globus_result_t                     result,
    int                                 debug,
    const char *                        filename,
    int                                 line)
{
    globus_object_t *                   error_obj;

    error_obj = globus_error_get(result);
    if(debug)
    {
        char *                          error_string = NULL;
        globus_libc_fprintf(stderr,
                            "\n%s:%d:",
                            filename ? filename : "(null)", 
                            line);
        error_string = globus_error_print_chain(error_obj);
        globus_libc_fprintf(stderr, "%s\n", error_string);
        if(error_string)
        {
            globus_libc_free(error_string);
        }
    }
    else 
    {
        globus_libc_fprintf(stderr,
                            "Use -debug for further information.\n\n");
    }
    globus_object_free(error_obj);
    globus_module_deactivate_all();
    exit(1);
}

PyObject *
assist_gridmap(char * globususerid)
{

    PyObject * tuple, py_userid_str;
    char * useridp; 
    int ret;

    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "assist_gridmap:: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_gss_assist_gridmap(globususerid, &useridp);
    if (ret != GLOBUS_SUCCESS)
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", useridp));
    }
    return tuple;

}

int
assist_userok(char * globususerid, char * userid)
{
    int ret;
    printf("globususerid: %s\n", globususerid); 
    printf("userid: %s\n", userid); 

    ret = globus_gss_assist_userok(globususerid, userid);
    return ret;
}

PyObject *
assist_map_local_user(char * local_user)
{
    PyObject * tuple;
    char * globusidp;
    int ret;
    
    if (!(tuple=PyTuple_New(2))) {
        PyErr_SetString(PyExc_RuntimeError, 
                        "assist_gridmap:: PyTuple_New()");
        return NULL;
    }
    
    ret = globus_gss_assist_gridmap(local_user, &globusidp);
    if (ret != GLOBUS_SUCCESS)
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", ret));
	Py_INCREF(Py_None);
	PyTuple_SET_ITEM(tuple, 1, Py_None);
    }
    else
    {
	PyTuple_SET_ITEM(tuple, 0, Py_BuildValue("i", GLOBUS_SUCCESS));
	PyTuple_SET_ITEM(tuple,1, Py_BuildValue("s", globusidp));
    }
    return tuple;

}


%}
